﻿<#
.NOTES
	===========================================================================
	 Modified on:   08.2020
	 Created by: 	HG
	 Organization: 	Layer2 GmbH
	 Version:		1.4
	===========================================================================

.DESCRIPTION
	Creates Cloud Connector connections from a specified existing connection template. 
	The connection template must be used to backup the Microsoft Teams Shared Document files 
	and save it to a local directory. 

.PARAMATERS
	$templateConnectionPath: Specify your Cloud Connector Microsoft Teams connection template path.
	$MFA: Multi-Factor authentication true or false.

.EXAMPLE 
	PowerShell Terminal: C:\> & '.\Layer2_MS_Teams_Connection_Creator.ps1' -templateConnectionPath 'C:\ProgramData\Layer2 Cloud Connector\Connections\TeamsDocuments-to-FileShare.xml' -MFA
#>

param
(
	[string]$templateConnectionPath = $null,
	[switch]$MFA
)

[System.Management.Automation.PSCredential]$Credentials = $null
$currentDirectory				 						= [System.IO.Path]::GetDirectoryName($myInvocation.MyCommand.Definition)
[string]$CloudConnectorConnectionsDirectory 			= "$($env:ProgramData)\Layer2 Cloud Connector\Connections"


$pnpName 					= 'SharePointPnPPowerShellOnline'
[string]$pnpVersion 		= '3.21.2005.2'
$pnpPath 					= Join-Path $currentDirectory $pnpName
$mstName 					= 'MicrosoftTeams'
[string]$msTeamsVersion 	= '1.0.6'
$mstPath 					= Join-Path $currentDirectory $mstName

if(!(Test-Path $pnpPath))
{
	Write-Host "Installing PowerShell 'PnP' & 'Microsoft Teams' Modules on the script current directory ' $currentDirectory '. This modules will not be installed on your system to avoid possible conflicts with existing modules" -ForegroundColor Yellow
	Save-Module -Name $pnpName -Path $currentDirectory -RequiredVersion $pnpVersion
}
Import-Module ("{0}\$pnpName\{1}\$pnpName.psd1" -f $currentDirectory, $pnpVersion) -DisableNameChecking

if(!(Test-Path $mstPath))
{
	Save-Module -Name $mstName -Path $currentDirectory -RequiredVersion $msTeamsVersion
}
Import-Module ("{0}\$mstName\{1}\$mstName.psd1" -f $currentDirectory, $msTeamsVersion) -DisableNameChecking

function New-CloudConnectorConnection
{
	param 
	(
		[xml]$xmlTemplate,
		[string]$connectionPathName,
		$entitiesWithChanges
	)
	
	[xml]$newConnection = $xmlTemplate

	$source = $entitiesWithChanges.Source
	$newConnection.connection.dataEntities.dataEntity[0].Attributes['connectionString'].Value = $source.ConnectionString

	$destination = $entitiesWithChanges.Destination
	$newConnection.connection.dataEntities.dataEntity[1].Attributes['connectionString'].Value = $destination.ConnectionString

	$newConnection.Save($connectionPathName)
	Write-Host "--------" -ForegroundColor Yellow
	Write-Host "Cloud Connector connection created successfully!:" -ForegroundColor Yellow
	Write-Host $connectionPathName -ForegroundColor Green
}

function Get-EntitiesConnectionString {
	param 
	(
		$xmlTemplate,
		[string]$templatePath
	)
	
	$entity1 = $xmlTemplate.connection.dataEntities.dataEntity[0]
	$entity2 = $xmlTemplate.connection.dataEntities.dataEntity[1]

	$type1 = $entity1.Attributes['type'].Value
	$type2 = $entity2.Attributes['type'].Value

	$name1 = $entity1.Attributes['name'].Value
	$name2 = $entity2.Attributes['name'].Value

	$connstr1 = $entity1.Attributes['connectionString'].Value
	$connstr2 = $entity2.Attributes['connectionString'].Value

	return @{
		$type1=@{
			'Name'=$name1;
			'Type'=$type1;
			'ConnectionString'=$connstr1
		};
		$type2=@{
			'Name'=$name2;
			'Type'=$type2;
			'ConnectionString'=$connstr2
		}
	}
}

# ============== Execution Starts ============== #

if(!$templateConnectionPath)
{
	$connectionCounter = 1
	$connections = (Get-ChildItem -Path $CloudConnectorConnectionsDirectory -Include *.xml -Recurse) | Select-Object @{ Name= "Id"; Expression = { (([ref]$connectionCounter).Value++) }}, Name

	Write-Host
	Write-Host "Id - Name"
	$connections | % { (Write-Host "$($_.Id) - " -ForegroundColor Yellow -NoNewline); (Write-Host $_.Name -ForegroundColor Green)  }
	Write-Host "--------" -ForegroundColor Yellow
	
	Write-Host "No template path was provided, please enter the connection Id from the list above that will be used as Template to generate your Cloud Connector Microsoft Team connections." -ForegroundColor Yellow
	$Id = Read-Host -Prompt "Enter Id"

	$connections | ForEach-Object {
		if($_.Id -eq $Id)
		{
			$templateConnectionPath = Join-Path $CloudConnectorConnectionsDirectory $_.Name
			Write-Host
			Write-Host "The provided template will be used $templateConnectionPath" -ForegroundColor Green
			Write-Host
			return
		}
	}
}

if(!(Test-Path -Path $templateConnectionPath))
{
	throw "The specified template path is not valid $($templateConnectionPath). Please verify!"
}

[xml]$xmlConnectionTemplate = Get-Content -Path $templateConnectionPath
$entities 					= (Get-EntitiesConnectionString -xmlTemplate $xmlConnectionTemplate -templatePath $templateConnectionPath)
$sourceEntity 				= $entities.source

Write-Host "Connecting to your Microsoft Teams..." -ForegroundColor Yellow
$azureContext = $null
if($MFA)
{
	$azureContext = Connect-MicrosoftTeams 
}
else 
{
	Write-Host ""
	Write-Host "Please provide your Email and Password to signup into your Office365 tenant..." -ForegroundColor Yellow
	$Credentials = Get-Credential -Message "Office365 Credentials"
	if(($null -eq $Credentials) -or !($Credentials.Password) -or !($Credentials.Username))
	{
		throw "The Office3665 credentials cannot be null or empty 'value', please re-run the script and provide valid credentials."		
	}

	try {
		$azureContext = Connect-MicrosoftTeams -Credentials $Credentials
	}
	catch {

		if($null -eq $azureContext)
		{
			Write-Host "--------" -ForegroundColor Yellow
			Write-Host "WARNING" -ForegroundColor Red
			Write-Host "Could not connect to your Office365 tenant with the provided credentials. Is your account set with Multifactor Authentication?"
			[string]$isMfa = Read-Host "y or n"
			if($isMfa -eq 'y')
			{
				$MFA = $true
				$azureContext = Connect-MicrosoftTeams 
			}

			Write-Host ""
		}

		if($null -eq $azureContext)
		{
			throw "Could not connect to your Office365 tenant. Please invoke this script with the Multifactor authentication parameter: -MFA. Check out the documentation for reference."
			exit
		}
	}
}

Write-Host "Successfully connected!" -ForegroundColor Green
$azureContext

Write-Host "Getting Teams..." -ForegroundColor Yellow
$teamCounter 	= 1
[string]$user 	= $azureContext.Account.Id
$teams = Get-Team -User $user | Select-Object @{ Name= "Id"; Expression = { (([ref]$teamCounter).Value++) }}, MailNickName

if($null -eq $teams)
{
	throw "Could not get the Microsoft Teams related to your user account $user"
	exit
}

Write-Host "--------" -ForegroundColor Yellow
Write-Host "ID - Team Name" -ForegroundColor Cyan
$teams | % { (Write-Host "$($_.Id) - " -ForegroundColor Yellow -NoNewline); (Write-Host $_.MailNickName -ForegroundColor Green)  }
Write-Host "--------" -ForegroundColor Yellow

Write-Host "Please enter the Team ID or IDs separated by coma(,). Leave empty and press Enter to select all Teams." -ForegroundColor Yellow
[string]$inputTeamId = Read-Host -Prompt "Enter Id" 
Write-Host ""

$selectedTeamChannelNames = @()
switch ($inputTeamId) {
	{ $_ -contains ',' } { 
		$teamIds = $inputTeamId.Split(',')
		foreach($teamId in $teamIds)
		{
			foreach($team in $teams)
			{
				if($team.Id -eq $teamId)
				{
					$selectedTeamChannelNames += $team.MailNickName
				}
			}
		}
	 }
	 { $_ -eq '' } {
		foreach($team in $teams)
		{
			$selectedTeamChannelNames += $team.MailNickName
		}
	 }
	Default {
		foreach($team in $teams)
		{
			if($team.Id -eq $inputTeamId)
			{
				$selectedTeamChannelNames += $team.MailNickName
				break
			}
		}
	}
}

[string]$sourceEntityconnectionString	= $sourceEntity.ConnectionString
$urlPattern 							= '([a-zA-Z]{3,})://([\w-]+\.)+[\w-]+(/[\w- ./?%&=]*)[^;]'
[uri]$stringUrl 						= Select-String -InputObject $sourceEntityconnectionString -Pattern $urlPattern -AllMatches | % { $_.Matches } | % { $_.Value }
$tenantUrl 								= "{0}://{1}" -f $stringUrl.Scheme, $stringUrl.Host
[string]$libNameFormAllItemsAspx 		= Select-String -InputObject $stringUrl.OriginalString -Pattern '[^\/]+(?=\/Forms\/AllItems\.aspx).+$' -AllMatches | % { $_.Matches } | % { $_.Value }

Write-Host "$($selectedTeamChannelNames.Count) Team(s) selected" -ForegroundColor Green
$selectedTeamChannelNames | ForEach-Object {
	$teamName = $_
	$teamSiteUrl = "{0}/sites/{1}" -f $tenantUrl, $teamName

	Write-Host "Creating Connection-string for the Team Site $teamSiteUrl ..." -ForegroundColor Yellow
	if($MFA)
	{
		Connect-PnPOnline -Url $teamSiteUrl -UseWebLogin
	}
	else 
	{
		Connect-PnPOnline -Url $teamSiteUrl -Credentials $Credentials
	}

	Write-Host "Getting the Team Shared Documents Library ..." -ForegroundColor Yellow
	Write-Host ""
	$sharedDocumentsLibrary = Get-PnPList -Identity $libNameFormAllItemsAspx
	if($null -eq $sharedDocumentsLibrary)
	{
		$libNameFormAllItemsAspx =  'Shared%20Documents/Forms/AllItems.aspx'
		$sharedDocumentsLibrary = Get-PnPList -Identity $libNameFormAllItemsAspx
	
		if($null -eq $sharedDocumentsLibrary)
		{
			Write-Host "We could not find the Shared Document Library on your Team Site $teamSiteUrl, this might be due your site region and language configuration" -ForegroundColor Yellow
			Write-Host "the English name for this library is 'Shared Documents', please specify your Microsoft Teams Shared Document name" -ForegroundColor Yellow
			[string]$inputLibName = Read-Host -Prompt "Your MS Teams Document Library name is"
			$libNameFormAllItemsAspx = $inputLibName.Replace(' ', '%20')

			$sharedDocumentsLibrary = Get-PnPList -Identity $libNameFormAllItemsAspx
			if($null -eq $sharedDocumentsLibrary)
			{
				throw "The MS Teams Shared Library could not be found on the team site $teamSiteUrl, please verify the library name and run the script again."
			}
		}
	}
	
	$destinationEntity		= $entities.destination
	$systemDirectory 		= Select-String -InputObject $destinationEntity.ConnectionString -Pattern '(?<=Directory=).*(?=\;)' -AllMatches | % { $_.Matches } | % { $_.Value }
	$newDirectoryPath 		= Join-Path $systemDirectory $teamName
	if(!(Test-Path -Path $newDirectoryPath))
	{
		Write-Host "Creating backup directory $newDirectoryPath ..." -ForegroundColor Yellow
		$createdDir = New-Item -Path $newDirectoryPath -ItemType Directory
		if($createdDir.Exists)
		{
			Write-Host "Directory $newDirectoryPath successfully created." -ForegroundColor Green
		}
	}

	$newDestinationEntity	= @{'Name'=$destinationEntity.Name;'ConnectionString'=$destinationEntity.ConnectionString.Replace($systemDirectory, $newDirectoryPath)}
	$newSourceEntity 		= @{'Name'=$sourceEntity.Name;'ConnectionString'=$sourceEntityconnectionString.Replace($stringUrl.OriginalString, ("{0}/{1}" -f $teamSiteUrl, $libNameFormAllItemsAspx))}
	$newEntities			= @{'Source'=$newSourceEntity; 'Destination'=$newDestinationEntity}
	$newConnectionPathName 	= "{0}\Microsoft Teams - {1}.xml" -f $CloudConnectorConnectionsDirectory, $teamName

	New-CloudConnectorConnection -xmlTemplate $xmlConnectionTemplate -connectionPathName $newConnectionPathName -entitiesWithChanges $newEntities 

	Start-Sleep -Seconds 5
}